#include <iostream>
#include <chrono>
#include <boost/asio.hpp>

class RepeatTimer
{
public:
    explicit RepeatTimer(boost::asio::strand<boost::asio::any_io_executor> strand, std::chrono::milliseconds every, std::function<void()> task)
    : m_timer(std::move(strand), every), m_every(every), m_task(task)
    {
        m_timer.async_wait([this](boost::system::error_code ec){ doRunOne(ec); });
    }
    ~RepeatTimer() = default; // destroying the timer cancels it, too
    
    void cancel() { m_timer.cancel(); }
    
private:
    void doRunOne(boost::system::error_code ec);
    
    boost::asio::steady_timer m_timer;
    std::chrono::milliseconds m_every;
    std::function<void()> m_task;
};

void RepeatTimer::doRunOne(boost::system::error_code ec)
{
    // we canceled or destroyed the timer
    if (ec == boost::asio::error::operation_aborted)
        return;
     // there was a problem
    else if (ec)
        throw std::runtime_error{"Repeat timer got error code " + std::to_string(ec.value())};

    // normal operation. Run our task and repeat
    m_timer.expires_after(m_every);
    m_task();
    m_timer.async_wait([this](boost::system::error_code ec){ doRunOne(ec); });
}

int main()
{
    int counter = 0;
    boost::asio::io_context ctx;
    RepeatTimer t{boost::asio::make_strand(ctx), std::chrono::seconds{1}, [&](){
        std::cerr << counter++ << std::endl;
    }};
    ctx.run_for(std::chrono::seconds{4});
}
